unit referee_test;

{$mode objfpc}{$H+}

interface

uses
   Forms, ExtCtrls, StdCtrls, referee_wedstrijd_sheet;

type
  TForm_Test = class(TForm)
    BT_Close: TButton;
    FullTextMemo: TMemo;
    PenaltyMemo: TMemo;
    Splitter: TSplitter;
    TestButton: TButton;
    TopPanel: TPanel;
    procedure BT_CloseClick(Sender: TObject);
    procedure TestButtonClick(Sender: TObject);
  private
    procedure PopulateMemosFromFile(aFilename: String);
  public
  end;

var
  Form_Test: TForm_Test;

implementation

{$R *.lfm}

function GetName(pp: TWedstrijdSheetParser; aTeam: TTeamStr): String;
begin
  case aTeam of
    sTeamA: Exit(pp.TeamA);
    sTeamB: Exit(pp.TeamB);
  end;
end;

procedure TForm_Test.TestButtonClick(Sender: TObject);
begin
  PopulateMemosFromFile('52702.txt');
end;

procedure TForm_Test.PopulateMemosFromFile(aFilename: String);
var
  pp: TWedstrijdSheetParser;
  team: TTeamStr;
  period: TPeriodStr;
  penaltyKind: TPenaltyKind;
  total: Integer;
  FlatString: string;
  MinorA, MajorA, MiscA, GMPA, MPA, PSA: integer;
  MinorB, MajorB, MiscB, GMPB, MPB, PSB: integer;


  function PeriodToStr(aPeriod: TPeriodStr): String;
  begin
    WriteStr(Result, aPeriod);
    Delete(Result, 1, 1);
  end;


  function PenaltyKindToStr(aPenaltyKind: TPenaltyKind): String;
  begin
    WriteStr(Result, aPenaltyKind);
    Delete(Result, 1, 2);
  end;


begin
  FullTextMemo.Lines.LoadFromFile(aFilename);


  pp := TWedstrijdSheetParser.Create(aFilename);


  try
    PenaltyMemo.Clear;
    PenaltyMemo.Append('Using data file "' + pp.Filename + '"'#10);
    MinorA := 0;              MinorB := 0;
    MajorA := 0;              MajorB := 0;
    MiscA := 0;               MiscB := 0;
    GMPA := 0;                GMPB := 0;
    MPA := 0;                 MPB := 0;
    PSA :=0;                  PSB :=0;


    for team := sTeamA to sTeamB do
      begin
        PenaltyMemo.Append(GetName(pp, team) + #10'-------------------------------');
//        total := 0;
        for period := Low(TPeriodStr) to High(TPeriodStr) do
          begin
//            PenaltyMemo.Append('    ' + PeriodToStr(period) + #10'    ------------');
            for penaltyKind := Low(TPenaltyKind) to High(TPenaltyKind) do
              begin
                total := pp.DataArray[team][period][penaltyKind];
                case penaltyKind of
                  pkMinor: if team = sTeamA then Inc(MinorA,total) else Inc(MinorB,total);
                  pkMajor: if team = sTeamA then Inc(MajorA,total) else Inc(MajorB,total);
                  pkMisc : if team = sTeamA then Inc(MiscA,total) else Inc(MiscB,total);
                  pkGMP  : if team = sTeamA then Inc(GMPA,total) else Inc(GMPB,total);
                  pkMP   : if team = sTeamA then Inc(MPA,total) else Inc(MPB,total);
                  pkPS   : if team = sTeamA then Inc(PSA,total) else Inc(PSB,total);
                end;  // case TPenaltyKind
//                WriteStr(FlatString, '"',GetName(pp,team),'","', Period,'","', PenaltyKind, '","');
                WriteStr(FlatString, '    ', GetName(pp,Team),'   ', Period,'   ', PenaltyKind, '   ', pp.DataArray[team][period][penaltyKind]);
//                WriteStr(FlatString, '    ', PenaltyKind, '   ', pp.DataArray[team][period][penaltyKind]);
                PenaltyMemo.Lines.Add (FlatString);
//               PenaltyMemo.Lines.Add('      %s: %d', [PenaltyKindToStr(penaltyKind), pp.DataArray[team][period][penaltyKind]]);
//                Inc(total, pp.DataArray[team][period][penaltyKind]);
              end;
//            PenaltyMemo.Lines.Add('      total penalties: %d'#10,[total]);
          end;
        PenaltyMemo.Append('');
      end;
    WriteStr(FlatString, '    ', GetName(pp,sTeamA),'   ', MinorA,'   ', MajorA, '   ', MiscA, '   ', GMPA, '   ', MPA, '   ', PSA);
    PenaltyMemo.Lines.Add (FlatString);
    WriteStr(FlatString, '    ', GetName(pp,sTeamB),'   ', MinorB,'   ', MajorB, '   ', MiscB, '   ', GMPB, '   ', MPB, '   ', PSB);
    PenaltyMemo.Lines.Add (FlatString);
  finally
    pp.Free;
  end;
end;

procedure TForm_Test.BT_CloseClick(Sender: TObject);
begin
  Close;
end;

end.

